/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import _ from 'lodash-es';
import { spring } from 'react-motion';
import { rgb } from 'd3-color';
import { scaleOrdinal, schemeCategory10, schemeCategory20, schemeCategory20b, schemeCategory20c } from 'd3-scale';
import { schemeAccent, schemeDark2, schemePaired, schemePastel1, schemePastel2, schemeSet1, schemeSet2, schemeSet3 } from 'd3-scale-chromatic';

export var getColorGenerator = function getColorGenerator(instruction) {
    if (instruction === 'none') {
        return 'none';
    }

    if (_.isFunction(instruction)) {
        return instruction;
    }

    if (instruction === 'inherit') {
        return function (d) {
            return d.color || d.data.color;
        };
    }

    var inheritMatches = instruction.match(/inherit:(darker|brighter)\(([0-9.]+)\)/);
    if (inheritMatches) {
        var method = inheritMatches[1];
        var amount = inheritMatches[2];

        return function (d) {
            return rgb(d.color || d.data.color)[method](parseFloat(amount));
        };
    }

    return function () {
        return instruction;
    };
};

export var getColorStyleObject = function getColorStyleObject(instruction, property) {
    var style = {};

    var color = getColorGenerator(instruction);
    if (color !== 'none') {
        style[property] = color;
    }

    return style;
};

export var extractRGB = function extractRGB(_color, springConfig) {
    var color = rgb(_color);

    if (!springConfig) {
        return {
            colorR: color.r,
            colorG: color.g,
            colorB: color.b
        };
    }

    return {
        colorR: spring(color.r, Object.assign({}, springConfig, { precision: 1 })),
        colorG: spring(color.g, Object.assign({}, springConfig, { precision: 1 })),
        colorB: spring(color.b, Object.assign({}, springConfig, { precision: 1 }))
    };
};

var d3Colors = {
    d310: scaleOrdinal(schemeCategory10),
    d320: scaleOrdinal(schemeCategory20),
    d320b: scaleOrdinal(schemeCategory20b),
    d320c: scaleOrdinal(schemeCategory20c),
    accent: scaleOrdinal(schemeAccent),
    dark2: scaleOrdinal(schemeDark2),
    paired: scaleOrdinal(schemePaired),
    pastel1: scaleOrdinal(schemePastel1),
    pastel2: scaleOrdinal(schemePastel2),
    set1: scaleOrdinal(schemeSet1),
    set2: scaleOrdinal(schemeSet2),
    set3: scaleOrdinal(schemeSet3)
};

export var nivoCategoricalColors = function nivoCategoricalColors() {
    return scaleOrdinal(['#e8c1a0', '#f47560', '#f1e15b', '#e8a838', '#61cdbb', '#97e3d5']);
};

var dataColor = function dataColor(d) {
    return d.color || d.data.color;
};

export var getColorRange = function getColorRange(instruction) {
    if (instruction === 'data') return dataColor;

    if (instruction === 'nivo') return nivoCategoricalColors();

    if (_.isFunction(instruction)) return instruction;

    if (d3Colors[instruction]) return d3Colors[instruction];

    if (_.isArray(instruction)) return scaleOrdinal(instruction);

    return function () {
        return instruction;
    };
};

export var getColorsGenerator = function getColorsGenerator(colors, colorBy) {
    // skip range, color should be bound to data
    if (_.isFunction(colorBy)) return function (d) {
        return colorBy(d);
    };

    var scale = void 0;
    var getColorId = function getColorId(d) {
        return _.get(d, colorBy);
    };

    if (colors === 'nivo') {
        // use default nivo categorical colors
        scale = nivoCategoricalColors();
    } else if (d3Colors[colors]) {
        // use predefined d3 colors
        scale = d3Colors[colors];
    } else if (_.isArray(colors)) {
        // user defined color range
        scale = scaleOrdinal(colors);
    } else {
        // just use provided value, all elements will have identical color
        return function (d) {
            return colors;
        };
    }

    return function (d) {
        return scale(getColorId(d));
    };
};

/**
 * Memoize both color generator & color generator result.
 */
var memoizedColorModifier = _.memoize(function (method, _amount) {
    var amount = parseFloat(_amount);

    return _.memoize(function (d) {
        return rgb(d.color)[method](amount);
    }, function (d) {
        return d.color;
    });
}, function (method, amount) {
    return method + '.' + amount;
});

var noneGenerator = function noneGenerator() {
    return 'none';
};
var inheritGenerator = function inheritGenerator(d) {
    return d.color;
};

/**
 * @param {string|Function} instruction
 * @param {string}          [themeKey]
 * @return {Function}
 */
export var getInheritedColorGenerator = function getInheritedColorGenerator(instruction, themeKey) {
    if (instruction === 'none') return noneGenerator;

    if (_.isFunction(instruction)) return instruction;

    if (instruction === 'theme') {
        return function (d, theme) {
            return _.get(theme, themeKey);
        };
    }

    if (instruction === 'inherit') return inheritGenerator;

    var inheritMatches = instruction.match(/inherit:(darker|brighter)\(([0-9.]+)\)/);
    if (inheritMatches) {
        var method = inheritMatches[1];
        var amount = inheritMatches[2];

        return memoizedColorModifier(method, amount);
    }

    return function () {
        return instruction;
    };
};