/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';
import compose from 'recompose/compose';
import defaultProps from 'recompose/defaultProps';
import pure from 'recompose/pure';
import { chord as d3Chord, ribbon as Ribbon } from 'd3-chord';
import { arc as Arc } from 'd3-shape';
import { rgb } from 'd3-color';
import { withTheme, withDimensions } from '../../../hocs';
import { getColorRange } from '../../../lib/colorUtils';
import Container from '../Container';
import SvgWrapper from '../SvgWrapper';

var Chord = function Chord(_ref) {
    var data = _ref.data,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        padAngle = _ref.padAngle,
        innerRadiusRatio = _ref.innerRadiusRatio,
        innerRadiusOffset = _ref.innerRadiusOffset,
        ribbonOpacity = _ref.ribbonOpacity,
        ribbonBorderWidth = _ref.ribbonBorderWidth,
        arcOpacity = _ref.arcOpacity,
        arcBorderWidth = _ref.arcBorderWidth,
        theme = _ref.theme,
        colors = _ref.colors,
        isInteractive = _ref.isInteractive;

    var centerX = width / 2;
    var centerY = height / 2;

    var color = getColorRange(colors);

    var radius = Math.min(width, height) / 2;
    var arcInnerRadius = radius * innerRadiusRatio;
    var ribbonRadius = radius * (innerRadiusRatio - innerRadiusOffset);

    var chord = d3Chord().padAngle(padAngle);

    var arc = Arc().innerRadius(arcInnerRadius).outerRadius(radius);

    var ribbon = Ribbon().radius(ribbonRadius);

    var ribbons = chord(data);
    var arcs = ribbons.groups;

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;

            return React.createElement(
                SvgWrapper,
                { width: outerWidth, height: outerHeight, margin: margin },
                React.createElement(
                    'g',
                    { transform: 'translate(' + centerX + ', ' + centerY + ')' },
                    React.createElement(
                        'g',
                        null,
                        ribbons.map(function (d) {
                            var c = rgb(color(d.source.index));
                            c = rgb(c.r, c.g, c.b, ribbonOpacity);

                            return React.createElement('path', {
                                key: 'ribbon.' + d.source.index + '.' + d.target.index,
                                className: 'nivo_chord_ribbon',
                                d: ribbon(d),
                                fill: c,
                                stroke: c,
                                strokeWidth: ribbonBorderWidth
                            });
                        })
                    ),
                    React.createElement(
                        'g',
                        null,
                        arcs.map(function (d) {
                            var c = rgb(color(d.index));
                            c = rgb(c.r, c.g, c.b, arcOpacity);

                            return React.createElement('path', {
                                key: 'arc.' + d.index,
                                className: 'nivo_chord_arc',
                                d: arc(d),
                                fill: c,
                                stroke: c,
                                strokeWidth: arcBorderWidth
                            });
                        })
                    )
                )
            );
        }
    );
};

Chord.propTypes = {
    data: PropTypes.array.isRequired,

    padAngle: PropTypes.number.isRequired,
    innerRadiusRatio: PropTypes.number.isRequired,
    innerRadiusOffset: PropTypes.number.isRequired,

    ribbonOpacity: PropTypes.number.isRequired,
    ribbonBorderWidth: PropTypes.number.isRequired,

    // colors
    colors: PropTypes.any.isRequired,

    // interactivity
    isInteractive: PropTypes.bool.isRequired
};

export var ChordDefaultProps = {
    padAngle: 0,
    innerRadiusRatio: 0.9,
    innerRadiusOffset: 0,

    ribbonOpacity: 0.5,
    ribbonBorderWidth: 1,

    arcOpacity: 1,
    arcBorderWidth: 1,

    // colors
    colors: 'nivo',

    // interactivity
    isInteractive: true
};

var enhance = compose(defaultProps(ChordDefaultProps), withTheme(), withDimensions(), pure);

export default enhance(Chord);