var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';
import { min, max, range, sortBy } from 'lodash-es';
import { stack as d3Stack, area } from 'd3-shape';
import { scaleLinear, scalePoint } from 'd3-scale';
import compose from 'recompose/compose';
import pure from 'recompose/pure';
import withPropsOnChange from 'recompose/withPropsOnChange';
import defaultProps from 'recompose/defaultProps';
import { areaCurvePropType, stackOrderPropType, stackOrderFromProp, stackOffsetPropType, stackOffsetFromProp } from '../../../props';
import { withTheme, withCurve, withDimensions, withMotion } from '../../../hocs';
import { getColorRange } from '../../../lib/colorUtils';
import SvgWrapper from '../SvgWrapper';
import Container from '../Container';
import Axes from '../../axes/Axes';
import Grid from '../../axes/Grid';
import StreamLayers from './StreamLayers';
import StreamSlices from './StreamSlices';

var stackMin = function stackMin(layers) {
    return min(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[0];
        }));
    }, []));
};
var stackMax = function stackMax(layers) {
    return max(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[1];
        }));
    }, []));
};

var Stream = function Stream(_ref) {
    var data = _ref.data,
        keys = _ref.keys,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        layers = _ref.layers,
        areaGenerator = _ref.areaGenerator,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        theme = _ref.theme,
        getColor = _ref.getColor,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive,
        enableStackTooltip = _ref.enableStackTooltip;

    var enhancedLayers = layers.map(function (points, i) {
        var layer = points.map(function (_ref2, i) {
            var y1 = _ref2[0],
                y2 = _ref2[1];
            return {
                index: i,
                value: y2 - y1,
                x: xScale(i),
                y1: yScale(y1),
                y2: yScale(y2)
            };
        });

        return {
            id: keys[i],
            layer: layer,
            path: areaGenerator(layer),
            color: getColor(i)
        };
    });

    var slices = range(data.length).map(function (i) {
        return {
            index: i,
            x: enhancedLayers[0].layer[i].x,
            stack: sortBy(enhancedLayers.map(function (layer) {
                return _extends({
                    id: layer.id,
                    color: layer.color
                }, layer.layer[i]);
            }), 'y2')
        };
    });

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;
            return React.createElement(
                SvgWrapper,
                { width: outerWidth, height: outerHeight, margin: margin },
                React.createElement(Grid, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                React.createElement(StreamLayers, _extends({
                    layers: enhancedLayers,
                    fillOpacity: fillOpacity,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip
                }, motionProps)),
                React.createElement(Axes, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                isInteractive && enableStackTooltip && React.createElement(StreamSlices, {
                    slices: slices,
                    height: height,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip,
                    theme: theme
                })
            );
        }
    );
};

Stream.propTypes = {
    // data
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    keys: PropTypes.array.isRequired,

    stack: PropTypes.func.isRequired,
    xScale: PropTypes.func.isRequired,
    yScale: PropTypes.func.isRequired,

    order: stackOrderPropType.isRequired,
    offsetType: stackOffsetPropType.isRequired,
    curve: areaCurvePropType.isRequired,
    areaGenerator: PropTypes.func.isRequired,

    // axes & grid
    axisTop: PropTypes.object,
    axisRight: PropTypes.object,
    axisBottom: PropTypes.object,
    axisLeft: PropTypes.object,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // theming
    colors: PropTypes.any.isRequired,
    fillOpacity: PropTypes.number.isRequired,
    getColor: PropTypes.func.isRequired,

    // interactivity
    isInteractive: PropTypes.bool,

    // stack tooltip
    enableStackTooltip: PropTypes.bool.isRequired
};

export var StreamDefaultProps = {
    order: 'none',
    offsetType: 'wiggle',
    curve: 'catmullRom',

    // axes & grid
    axisBottom: {},
    enableGridX: true,
    enableGridY: false,

    // theming
    colors: 'nivo',
    fillOpacity: 1,

    // interactivity
    isInteractive: true,

    // stack tooltip
    enableStackTooltip: true
};

var enhance = compose(defaultProps(StreamDefaultProps), withTheme(), withCurve(), withDimensions(), withMotion(), withPropsOnChange(['curveInterpolator'], function (_ref4) {
    var curveInterpolator = _ref4.curveInterpolator;
    return {
        areaGenerator: area().x(function (_ref5) {
            var x = _ref5.x;
            return x;
        }).y0(function (_ref6) {
            var y1 = _ref6.y1;
            return y1;
        }).y1(function (_ref7) {
            var y2 = _ref7.y2;
            return y2;
        }).curve(curveInterpolator)
    };
}), withPropsOnChange(['colors'], function (_ref8) {
    var colors = _ref8.colors;
    return {
        getColor: getColorRange(colors)
    };
}), withPropsOnChange(['keys', 'offsetType', 'order'], function (_ref9) {
    var keys = _ref9.keys,
        offsetType = _ref9.offsetType,
        order = _ref9.order;
    return {
        stack: d3Stack().keys(keys).offset(stackOffsetFromProp(offsetType)).order(stackOrderFromProp(order))
    };
}), withPropsOnChange(['stack', 'data', 'width', 'height'], function (_ref10) {
    var stack = _ref10.stack,
        data = _ref10.data,
        width = _ref10.width,
        height = _ref10.height;

    var layers = stack(data);

    var minValue = stackMin(layers);
    var maxValue = stackMax(layers);

    return {
        layers: layers,
        xScale: scalePoint().domain(range(data.length)).range([0, width]),
        yScale: scaleLinear().domain([minValue, maxValue]).range([height, 0])
    };
}), pure);

export default enhance(Stream);