var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import { range, max, maxBy, sumBy, uniq } from 'lodash-es';
import { scaleBand, scaleLinear } from 'd3-scale';

/**
 * Generates X scale.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @param {number}         padding
 * @returns {Function}
 */
export var getXScale = function getXScale(data, width, padding) {
    var xLengths = uniq(data.map(function (_ref) {
        var data = _ref.data;
        return data.length;
    }));
    if (xLengths.length > 1) {
        throw new Error(['Found inconsitent data for x,', 'expecting all series to have same length', 'but found: ' + xLengths.join(', ')].join(' '));
    }

    return scaleBand().rangeRound([0, width]).domain(data[0].data.map(function (_ref2) {
        var x = _ref2.x;
        return x;
    })).padding(padding);
};

/**
 * Generates Y scale for grouped bar chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         height
 * @returns {Function}
 */
export var getGroupedYScale = function getGroupedYScale(data, height) {
    var maxY = maxBy(data.reduce(function (acc, serie) {
        return [].concat(acc, serie.data);
    }, []), 'y').y;

    return scaleLinear().rangeRound([height, 0]).domain([0, maxY]);
};

/**
 * Generates Y scale for stacked bar chart.
 *
 * @param {Array.<Object>} data
 * @param {Object}         xScale
 * @param {number}         height
 */
export var getStackedYScale = function getStackedYScale(data, xScale, height) {
    var maxY = max(range(xScale.domain().length).map(function (i) {
        return sumBy(data, function (serie) {
            return serie.data[i].y;
        });
    }));

    return scaleLinear().rangeRound([height, 0]).domain([0, maxY]);
};

/**
 * Generates x/y scales & bars for grouped bar chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @param {number}         height
 * @param {Function}       color
 * @param {number}         xPadding
 * @return {{ xScale: Function, yScale: Function, bars: Array.<Object> }}
 */
export var generateGroupedBars = function generateGroupedBars(data, width, height, color) {
    var _ref3 = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {},
        _ref3$xPadding = _ref3.xPadding,
        xPadding = _ref3$xPadding === undefined ? 0 : _ref3$xPadding;

    var xScale = getXScale(data, width, xPadding);
    var yScale = getGroupedYScale(data, height);

    var bars = [];
    data.forEach(function (serie, serieIndex) {
        serie.data.forEach(function (d) {
            var barWidth = xScale.bandwidth() / data.length;
            var x = xScale(d.x) + barWidth * serieIndex;
            var y = yScale(d.y);
            var barHeight = height - y;

            var value = d.y;

            if (barWidth > 0 && barHeight > 0) {
                bars.push({
                    key: serie.id + '.' + d.x,
                    value: value,
                    x: x,
                    y: y,
                    width: barWidth,
                    height: barHeight,
                    color: color(_extends({}, d, { serie: serie }))
                });
            }
        });
    });

    return { xScale: xScale, yScale: yScale, bars: bars };
};

/**
 * Generates x/y scales & bars for stacked bar chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @param {number}         height
 * @param {Function}       color
 * @param {number}         xPadding
 * @return {{ xScale: Function, yScale: Function, bars: Array.<Object> }}
 */
export var generateStackedBars = function generateStackedBars(data, width, height, color) {
    var _ref4 = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {},
        _ref4$xPadding = _ref4.xPadding,
        xPadding = _ref4$xPadding === undefined ? 0 : _ref4$xPadding;

    var xScale = getXScale(data, width, xPadding);
    var yScale = getStackedYScale(data, xScale, height);

    var stackedData = data.map(function (serie) {
        return _extends({}, serie, {
            data: []
        });
    });

    range(xScale.domain().length).forEach(function (__, index) {
        data.forEach(function (_ref5, serieIndex) {
            var serie = _ref5.data;

            var d = serie[index];

            var y0 = 0;
            var y1 = d.y;
            if (serieIndex > 0) {
                y0 = stackedData[serieIndex - 1].data[index].y1;
                y1 = d.y + y0;
            }

            stackedData[serieIndex].data[index] = Object.assign({}, d, {
                y0: y0,
                y1: y1
            });
        });
    });

    var bars = [];
    stackedData.forEach(function (serie) {
        serie.data.forEach(function (d) {
            var x = xScale(d.x);
            var barWidth = xScale.bandwidth();
            var y = yScale(d.y1);
            var barHeight = yScale(d.y0) - y;

            var value = d.y;

            if (barWidth > 0 && barHeight > 0) {
                bars.push({
                    key: serie.id + '.' + d.x,
                    value: value,
                    x: x,
                    y: y,
                    width: barWidth,
                    height: barHeight,
                    color: color(_extends({}, d, { serie: serie }))
                });
            }
        });
    });

    return { xScale: xScale, yScale: yScale, bars: bars };
};