/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import d3 from 'd3';
import { degreesToRadians, midAngle, findNeighbor } from '../../../lib/arcUtils';
import { getColorRange } from '../../../lib/colorUtils';

var PieD3Svg = function PieD3Svg(domRoot) {
    // DOM elements
    var element = d3.select(domRoot);
    var outline = element.append('path').attr('class', 'nivo_pie_outline');
    var slices = element.append('g').attr('class', 'nivo_pie_slices');

    // d3 generators
    var pie = d3.layout.pie();
    var arc = d3.svg.arc();

    // used to store previous state for smart transitions
    var previousData = [];

    // an array to store decorator functions
    var decorators = [];

    return {
        draw: function draw(props) {
            var data = props.data,
                width = props.width,
                height = props.height,
                sort = props.sort,
                keyProp = props.keyProp,
                valueProp = props.valueProp,
                startAngle = props.startAngle,
                endAngle = props.endAngle,
                padAngle = props.padAngle,
                cornerRadius = props.cornerRadius,
                colors = props.colors,
                innerRadius = props.innerRadius,
                transitionDuration = props.transitionDuration,
                transitionEasing = props.transitionEasing;


            var identity = function identity(d) {
                return d.data[keyProp];
            };
            var color = getColorRange(colors);

            element.attr('transform', 'translate(' + width / 2 + ', ' + height / 2 + ')');

            pie.sort(sort).value(function (d) {
                return d[valueProp];
            }).startAngle(degreesToRadians(startAngle)).endAngle(degreesToRadians(endAngle)).padAngle(degreesToRadians(padAngle));

            var radius = Math.min(width / 2, height / 2);

            arc.outerRadius(radius).innerRadius(radius * innerRadius).cornerRadius(cornerRadius);

            outline.attr('d', arc({
                startAngle: degreesToRadians(startAngle),
                endAngle: degreesToRadians(endAngle)
            }));

            var slice = slices.selectAll('.nivo_pie_slice');

            var newData = pie(data.map(function (d, i) {
                if (!d.color) {
                    d.color = color(i);
                }

                return d;
            }));

            //console.log(_.cloneDeep(newData));

            function arcTween(a) {
                var i = d3.interpolate(this._current, a);
                this._current = i(0);

                return function (t) {
                    return arc(i(t));
                };
            }

            slice = slice.data(newData, identity);

            // —————————————————————————————————————————————————————————————————————————————————————————————————————————
            // ENTER: creates new elements
            // —————————————————————————————————————————————————————————————————————————————————————————————————————————
            slice.enter().append('path').attr('class', 'nivo_pie_slice').style('fill', function (d) {
                return d.data.color;
            }).each(function (d, i) {
                var angle = midAngle(d);

                this._current = findNeighbor(i, identity, previousData, newData) || _.assign({}, d, {
                    startAngle: 0,
                    endAngle: 0
                });
            });

            // —————————————————————————————————————————————————————————————————————————————————————————————————————————
            // UPDATE: updates existing elements
            // —————————————————————————————————————————————————————————————————————————————————————————————————————————
            slice.transition().duration(transitionDuration).ease(transitionEasing).style('fill', function (d) {
                return d.data.color;
            }).attrTween('d', arcTween);

            // —————————————————————————————————————————————————————————————————————————————————————————————————————————
            // EXIT: removes stale elements
            // —————————————————————————————————————————————————————————————————————————————————————————————————————————
            slice.exit().attr('class', 'nivo_exit').datum(function (d, i) {
                return findNeighbor(i, identity, newData, previousData) || d;
            }).transition().duration(transitionDuration).ease(transitionEasing).attrTween('d', arcTween).remove();

            var pieContext = {
                element: element,
                pie: pie,
                arc: arc,
                radius: radius,
                identity: identity,
                previousData: previousData,
                newData: newData,
                transitionDuration: transitionDuration,
                transitionEasing: transitionEasing
            };

            decorators.forEach(function (legend) {
                legend(pieContext);
            });

            previousData = newData;
        },
        decorate: function decorate() {
            var newDecorators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

            decorators = newDecorators;
        }
    };
};

export default PieD3Svg;