'use strict';

exports.__esModule = true;
exports.getInheritedColorGenerator = exports.getColorsGenerator = exports.getColorRange = exports.nivoCategoricalColors = exports.extractRGB = exports.getColorStyleObject = exports.getColorGenerator = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _reactMotion = require('react-motion');

var _d3Color = require('d3-color');

var _d3Scale = require('d3-scale');

var _d3ScaleChromatic = require('d3-scale-chromatic');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getColorGenerator = exports.getColorGenerator = function getColorGenerator(instruction) {
    if (instruction === 'none') {
        return 'none';
    }

    if (_lodash2.default.isFunction(instruction)) {
        return instruction;
    }

    if (instruction === 'inherit') {
        return function (d) {
            return d.color || d.data.color;
        };
    }

    var inheritMatches = instruction.match(/inherit:(darker|brighter)\(([0-9.]+)\)/);
    if (inheritMatches) {
        var method = inheritMatches[1];
        var amount = inheritMatches[2];

        return function (d) {
            return (0, _d3Color.rgb)(d.color || d.data.color)[method](parseFloat(amount));
        };
    }

    return function () {
        return instruction;
    };
}; /*
    * This file is part of the nivo project.
    *
    * Copyright 2016-present, Raphaël Benitte.
    *
    * For the full copyright and license information, please view the LICENSE
    * file that was distributed with this source code.
    */
var getColorStyleObject = exports.getColorStyleObject = function getColorStyleObject(instruction, property) {
    var style = {};

    var color = getColorGenerator(instruction);
    if (color !== 'none') {
        style[property] = color;
    }

    return style;
};

var extractRGB = exports.extractRGB = function extractRGB(_color, springConfig) {
    var color = (0, _d3Color.rgb)(_color);

    if (!springConfig) {
        return {
            colorR: color.r,
            colorG: color.g,
            colorB: color.b
        };
    }

    return {
        colorR: (0, _reactMotion.spring)(color.r, Object.assign({}, springConfig, { precision: 1 })),
        colorG: (0, _reactMotion.spring)(color.g, Object.assign({}, springConfig, { precision: 1 })),
        colorB: (0, _reactMotion.spring)(color.b, Object.assign({}, springConfig, { precision: 1 }))
    };
};

var d3Colors = {
    d310: (0, _d3Scale.scaleOrdinal)(_d3Scale.schemeCategory10),
    d320: (0, _d3Scale.scaleOrdinal)(_d3Scale.schemeCategory20),
    d320b: (0, _d3Scale.scaleOrdinal)(_d3Scale.schemeCategory20b),
    d320c: (0, _d3Scale.scaleOrdinal)(_d3Scale.schemeCategory20c),
    accent: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemeAccent),
    dark2: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemeDark2),
    paired: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemePaired),
    pastel1: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemePastel1),
    pastel2: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemePastel2),
    set1: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemeSet1),
    set2: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemeSet2),
    set3: (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemeSet3)
};

var nivoCategoricalColors = exports.nivoCategoricalColors = function nivoCategoricalColors() {
    return (0, _d3Scale.scaleOrdinal)(['#e8c1a0', '#f47560', '#f1e15b', '#e8a838', '#61cdbb', '#97e3d5']);
};

var dataColor = function dataColor(d) {
    return d.color || d.data.color;
};

var getColorRange = exports.getColorRange = function getColorRange(instruction) {
    if (instruction === 'data') return dataColor;

    if (instruction === 'nivo') return nivoCategoricalColors();

    if (_lodash2.default.isFunction(instruction)) return instruction;

    if (d3Colors[instruction]) return d3Colors[instruction];

    if (_lodash2.default.isArray(instruction)) return (0, _d3Scale.scaleOrdinal)(instruction);

    return function () {
        return instruction;
    };
};

var getColorsGenerator = exports.getColorsGenerator = function getColorsGenerator(colors, colorBy) {
    // skip range, color should be bound to data
    if (_lodash2.default.isFunction(colorBy)) return function (d) {
        return colorBy(d);
    };

    var scale = void 0;
    var getColorId = function getColorId(d) {
        return _lodash2.default.get(d, colorBy);
    };

    if (colors === 'nivo') {
        // use default nivo categorical colors
        scale = nivoCategoricalColors();
    } else if (d3Colors[colors]) {
        // use predefined d3 colors
        scale = d3Colors[colors];
    } else if (_lodash2.default.isArray(colors)) {
        // user defined color range
        scale = (0, _d3Scale.scaleOrdinal)(colors);
    } else {
        // just use provided value, all elements will have identical color
        return function (d) {
            return colors;
        };
    }

    return function (d) {
        return scale(getColorId(d));
    };
};

/**
 * Memoize both color generator & color generator result.
 */
var memoizedColorModifier = _lodash2.default.memoize(function (method, _amount) {
    var amount = parseFloat(_amount);

    return _lodash2.default.memoize(function (d) {
        return (0, _d3Color.rgb)(d.color)[method](amount);
    }, function (d) {
        return d.color;
    });
}, function (method, amount) {
    return method + '.' + amount;
});

var noneGenerator = function noneGenerator() {
    return 'none';
};
var inheritGenerator = function inheritGenerator(d) {
    return d.color;
};

/**
 * @param {string|Function} instruction
 * @param {string}          [themeKey]
 * @return {Function}
 */
var getInheritedColorGenerator = exports.getInheritedColorGenerator = function getInheritedColorGenerator(instruction, themeKey) {
    if (instruction === 'none') return noneGenerator;

    if (_lodash2.default.isFunction(instruction)) return instruction;

    if (instruction === 'theme') {
        return function (d, theme) {
            return _lodash2.default.get(theme, themeKey);
        };
    }

    if (instruction === 'inherit') return inheritGenerator;

    var inheritMatches = instruction.match(/inherit:(darker|brighter)\(([0-9.]+)\)/);
    if (inheritMatches) {
        var method = inheritMatches[1];
        var amount = inheritMatches[2];

        return memoizedColorModifier(method, amount);
    }

    return function () {
        return instruction;
    };
};